#!/bin/sh
# Copyright (c) 2000-2015 Synology Inc. All rights reserved.

######################################################################################################################
# script to generate syslog-ng relative configurations,
#
#   - Archive       | $LOGCENTER_TARGET/etc/syslog-ng/patterndb.d/pkg-LogCenter-localarchive.conf  | control by "localarch" option of this script
#   - Recv Server   | $LOGCENTER_TARGET/etc/syslog-ng/patterndb.d/pkg-LogCenter-recvrule.conf     | control by "recvrule" option of this script
#   - Client        | $LOGCENTER_TARGET/etc/syslog-ng/patterndb.d/pkg-LogCenter-client.conf   | control by "client" option of this script
#
######################################################################################################################
LOGCENTER_TARGET=/var/packages/LogCenter/target
SYSLOGNG_CONFIG_FOLDER=${LOGCENTER_TARGET}/etc/syslog-ng/patterndb.d
SYSLOGNG_CONFIG_COMMON=${SYSLOGNG_CONFIG_FOLDER}/common.conf
SYSLOGNG_CONFIG_ARCHIVE=${SYSLOGNG_CONFIG_FOLDER}/pkg-LogCenter-localarchive.conf
SYSLOGNG_CONFIG_RECVRULE=${SYSLOGNG_CONFIG_FOLDER}/pkg-LogCenter-recvrule.conf
SYSLOGNG_CONFIG_CLIENT=${SYSLOGNG_CONFIG_FOLDER}/pkg-LogCenter-client.conf
SYSLOGNG_CONFIG_NOTIFICATION=${SYSLOGNG_CONFIG_FOLDER}/syslog-notification.conf

SYSLOGNG_SERVER_SETTING=${LOGCENTER_TARGET}/service/conf/server.conf
SYSLOGNG_RECVRULE_SETTING=${LOGCENTER_TARGET}/service/conf/recvrule.conf
SYSLOGNG_CLIENT_SETTING=${LOGCENTER_TARGET}/service/conf/client.conf
SYSLOGNG_NOTIFY_SETTING=/etc/synosyslog/notify.conf

SYSLOGNG_TEMPLATE_FOLDER=${LOGCENTER_TARGET}/service/db_template
SYSLOGNG_ARCHIVE_TEMPLATE_FOLDER=${SYSLOGNG_TEMPLATE_FOLDER}/archive
SYSLOGNG_SERVER_TEMPLATE_FOLDER=${SYSLOGNG_TEMPLATE_FOLDER}/server
SYSLOGNG_CUSTRULE_TEMPLATE_FOLDER=${SYSLOGNG_TEMPLATE_FOLDER}/custrule
SYSLOGNG_CLIENT_TEMPLATE_FOLDER=${SYSLOGNG_TEMPLATE_FOLDER}/client

SYSLOGNG_CONFIG_NOTIFICATION=${SYSLOGNG_CONFIG_FOLDER}/syslog-notification.conf

SYSLOGNG_SCRIPT_FOLDER=${LOGCENTER_TARGET}/service/scripts
. ${SYSLOGNG_SCRIPT_FOLDER}/syslog_common.sh
. ${SYSLOGNG_SCRIPT_FOLDER}/syslog_client_util.sh
. ${SYSLOGNG_SCRIPT_FOLDER}/syslog_server_util.sh
. ${SYSLOGNG_SCRIPT_FOLDER}/syslog_localarch_util.sh
. ${SYSLOGNG_SCRIPT_FOLDER}/syslog_notification.sh

Usage(){
    echo "Usage: $0 (localarch|recvrule|client|notification) (confgen|confclr)"
}

case $1 in
    notification)
        case $2 in
            confgen)
                SyslogNotificationConfigGen
            ;;
            confclr)
                SyslogNotificationConfigClr
            ;;
            *)
                Usage
            ;;
        esac
    ;;
    localarch)
        case $2 in
            confgen)
                SyslogLocalarchConfigGen
            ;;
            confclr)
                SyslogLocalarchConfigClr
            ;;
            *)
                Usage
            ;;
        esac
    ;;
    recvrule)
        case $2 in
            confgen)
                RecvRuleConfigGen
            ;;
            confclr)
                RecvRuleConfigClr
            ;;
            *)
                Usage
            ;;
        esac
    ;;
    client)
    case $2 in
            confgen)
                SyslogClientConfigGen
            ;;
            confclr)
                SyslogClientConfigClr
            ;;
            *)
                Usage
            ;;
        esac
    ;;
    common)
        case $2 in
            confgen)
                SyslogCommonConfigGen
            ;;
            *)
                Usage
            ;;
        esac
    ;;
    *)
        Usage
    ;;
esac

