#!/bin/sh
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

BIN_GET_SKV=/usr/syno/bin/get_section_key_value
SYSLOGNG_TEMPLATE_SRC=${SYSLOGNG_CUSTRULE_TEMPLATE_FOLDER}/src.template
SYSLOGNG_TEMPLATE_PARSER=${SYSLOGNG_CUSTRULE_TEMPLATE_FOLDER}/parser.template
SYSLOGNG_TEMPLATE_PATTERN_DB=${SYSLOGNG_CUSTRULE_TEMPLATE_FOLDER}/pattern.template
SYSLOGNG_FOLDER_PATTERN_DB_XML=${LOGCENTER_TARGET}/usr/syno/etc/synosyslog/patterndb/
SYSLOGNG_SERVER_TLS="tls(key_file(\"/var/packages/LogCenter/target/service/conf/keys/ca.key\") cert_file(\"/var/packages/LogCenter/target/service/conf/keys/ca.crt\") peer_verify(optional-untrusted))"

########################
SyslogServerSettingGet(){
    if [ ! -f "${SYSLOGNG_SERVER_SETTING}" ]; then
        echo "Syslog setting file does not exist!"
        exit 1
    fi

    SYSLOGNG_SERVER_ARCH_DEST=`/bin/get_key_value ${SYSLOGNG_SERVER_SETTING} arch_dest`
    SYSLOGNG_SERVER_ARCH_BYDEVICE=`/bin/get_key_value ${SYSLOGNG_SERVER_SETTING} arch_by_device`

    if [ "" == "${SYSLOGNG_SERVER_ARCH_DEST}" ]; then
        exit 0
    fi
    if [ ! -e "${SYSLOGNG_SERVER_ARCH_DEST}" ]; then
        echo "(${LOCALARCH_DEST}) does not exist!";
        exit 1
    fi
}
####################################################

RecvRuleConfigGen(){
    SyslogServerSettingGet

    RecvRuleConfDataClear
    touch ${SYSLOGNG_CONFIG_RECVRULE}
    RecvRuleConfGen
}

RecvRuleConfigClr(){
    RecvRuleConfDataClear
}

########################
RecvRuleConfDataClear(){
    rm -rf ${SYSLOGNG_CONFIG_RECVRULE} > /dev/null 2>&1
}

RecvRuleConfGen() {
    # for each section, read config and gen one rule
    local RULE_SECS=`grep '^\[.*\]$' ${SYSLOGNG_RECVRULE_SETTING} | sed 's|^\[\(.*\)]$|\1|g'`
    local RULE=

    IetfFilterProcess
    for RULE in ${RULE_SECS}
    do
        RULE_NAME=${RULE}
        RULE_FORMAT=`${BIN_GET_SKV} ${SYSLOGNG_RECVRULE_SETTING} ${RULE_NAME} format`
        RULE_ENABLE=`${BIN_GET_SKV} ${SYSLOGNG_RECVRULE_SETTING} ${RULE_NAME} enable`
        RULE_PROTOCOL=`${BIN_GET_SKV} ${SYSLOGNG_RECVRULE_SETTING} ${RULE_NAME} protocol`
        RULE_PORT=`${BIN_GET_SKV} ${SYSLOGNG_RECVRULE_SETTING} ${RULE_NAME} port`
        RULE_SSL=`${BIN_GET_SKV} ${SYSLOGNG_RECVRULE_SETTING} ${RULE_NAME} ssl`
        if [ "bsd" == "${RULE_FORMAT}" ]; then
            BSDSingleRuleGen
        elif [ "ietf" == "${RULE_FORMAT}" ]; then
            IETFSingleRuleGen
        else
            CustomSingleRuleGen
        fi
    done
}

BSDSingleRuleGen() {
    if [ "yes" != ${RULE_ENABLE} ]; then
        return
    fi

    BsdSourceProcess
    BsdDestProcess
    BsdLogProcess
}

BsdSourceProcess(){
    local TEMPLATE_FILE=${SYSLOGNG_SERVER_TEMPLATE_FOLDER}/src.template
    local SOCKET_BUFFER=`cat /proc/sys/net/ipv4/tcp_wmem | awk '{print $3}'`
    local TRANSPORT="udp"
    local TLS_RULE=""

    #for VDSM
    if [ -z "${SOCKET_BUFFER}" ]; then
        SOCKET_BUFFER="4194304"
    fi

    if [ "tcp" == "${RULE_PROTOCOL}" ]; then
        TRANSPORT="tcp"
        if [ "yes" == "${RULE_SSL}" ]; then
            TRANSPORT="tls"
            TLS_RULE=${SYSLOGNG_SERVER_TLS}
        fi
    fi

    # process v4 source
    cat ${TEMPLATE_FILE} |
    sed -e "s/__SYNO_SRCNAME__/bsd_${RULE_NAME}/" \
        -e "s/__SYNO_SRCTYPE__/network/" \
        -e "s/__SYNO_ANYADDR__/\"0.0.0.0\"/" \
        -e "s/__SYNO_TRANSPORT__/${TRANSPORT}/" \
        -e "s/__SYNO_PORT__/${RULE_PORT}/" \
        -e "s/__SYNO_IPV__/4/" \
        -e "s=__SYNO_CA_SETTING__=${TLS_RULE}=" \
        -e "s/__SYNO_SOCKBUF__/${SOCKET_BUFFER}/" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}

    # process v6 source
    cat ${TEMPLATE_FILE} |
    sed -e "s/__SYNO_SRCNAME__/bsd_${RULE_NAME}_6/" \
        -e "s/__SYNO_SRCTYPE__/network/" \
        -e "s/__SYNO_ANYADDR__/\"::\"/" \
        -e "s/__SYNO_TRANSPORT__/${TRANSPORT}/" \
        -e "s/__SYNO_PORT__/${RULE_PORT}/" \
        -e "s/__SYNO_IPV__/6/" \
        -e "s=__SYNO_CA_SETTING__=${TLS_RULE}=" \
        -e "s/__SYNO_SOCKBUF__/${SOCKET_BUFFER}/" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}
}

BsdDestProcess(){
    local DB_PATH=
    if [ "${SYSLOGNG_SERVER_ARCH_BYDEVICE}" -eq 1 ]; then
        DB_PATH="${SYSLOGNG_SERVER_ARCH_DEST}/\$HOST/SYNOSYSLOGDB_\$HOST.DB"
    else
        DB_PATH="${SYSLOGNG_SERVER_ARCH_DEST}/SYNOSYSLOGDB__ARCH.DB"
    fi

    sed "s=__SYNO_DB_PATH__=${DB_PATH}=; s=__SYNO_RULE_NAME__=all_device=" \
        "${SYSLOGNG_SERVER_TEMPLATE_FOLDER}/arch_dest_db.template" >> ${SYSLOGNG_CONFIG_RECVRULE}
}

BsdLogProcess(){
    echo "log { source(s_syno_bsd_${RULE_NAME}); source(s_syno_bsd_${RULE_NAME}_6); flags(flow_control); };" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}
    echo "log { source(s_syno_bsd_${RULE_NAME}); source(s_syno_bsd_${RULE_NAME}_6); destination(d_syno_dest_recv); flags(flow_control); };" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}
    echo "log { source(s_syno_bsd_${RULE_NAME}); source(s_syno_bsd_${RULE_NAME}_6); destination(d_syno_arch_all_device_db); flags(flow_control); };" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}
}

IETFSingleRuleGen() {
    if [ "yes" != ${RULE_ENABLE} ]; then
        return
    fi

    IetfSourceProcess
    IetfDestProcess
    IetfLogProcess
}


IetfSourceProcess(){
    local TEMPLATE_FILE=${SYSLOGNG_SERVER_TEMPLATE_FOLDER}/src.template
    local SOCKET_BUFFER=`cat /proc/sys/net/ipv4/tcp_wmem | awk '{print $3}'`
    local TRANSPORT="udp"
    local TLS_RULE=""

    if [ -z "${SOCKET_BUFFER}" ]; then
        SOCKET_BUFFER="4194304"
    fi

    if [ "tcp" == "${RULE_PROTOCOL}" ]; then
        TRANSPORT="tcp"
        if [ "yes" == "${RULE_SSL}" ]; then
            TRANSPORT="tls"
            TLS_RULE=${SYSLOGNG_SERVER_TLS}
        fi
    fi

    # process v4 source
    cat ${TEMPLATE_FILE} |
    sed -e "s/__SYNO_SRCNAME__/ietf_${RULE_NAME}/" \
        -e "s/__SYNO_SRCTYPE__/syslog/" \
        -e "s/__SYNO_ANYADDR__/\"0.0.0.0\"/" \
        -e "s/__SYNO_TRANSPORT__/${TRANSPORT}/" \
        -e "s/__SYNO_PORT__/${RULE_PORT}/" \
        -e "s/__SYNO_IPV__/4/" \
        -e "s=__SYNO_CA_SETTING__=${TLS_RULE}=" \
        -e "s/__SYNO_SOCKBUF__/${SOCKET_BUFFER}/" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}

    # process v6 source
    cat ${TEMPLATE_FILE} |
    sed -e "s/__SYNO_SRCNAME__/ietf_${RULE_NAME}_6/" \
        -e "s/__SYNO_SRCTYPE__/syslog/" \
        -e "s/__SYNO_ANYADDR__/\"::\"/" \
        -e "s/__SYNO_TRANSPORT__/${TRANSPORT}/" \
        -e "s/__SYNO_PORT__/${RULE_PORT}/" \
        -e "s/__SYNO_IPV__/6/" \
        -e "s=__SYNO_CA_SETTING__=${TLS_RULE}=" \
        -e "s/__SYNO_SOCKBUF__/${SOCKET_BUFFER}/" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}
}

GeneralFilterProcess(){
    echo "filter f_syno_general { match('^System$' value('.SDATA.synolog@6574.synotype'));};" \
    >> ${SYSLOGNG_CONFIG_RECVRULE}
}
ConnectionFilterProcess(){
    echo "filter f_syno_connection { match('^Connection$' value('.SDATA.synolog@6574.synotype'));};" \
    >> ${SYSLOGNG_CONFIG_RECVRULE}
}
TransferFilterProcess(){
    echo "filter f_syno_ftpxfer { match('^FtpFileTransfer$' value('.SDATA.synolog@6574.synotype')); };" \
    >> ${SYSLOGNG_CONFIG_RECVRULE}
    echo "filter f_syno_fsxfer { match('^FileStation$' value('.SDATA.synolog@6574.synotype')); };" \
    >> ${SYSLOGNG_CONFIG_RECVRULE}
    echo "filter f_syno_fbxfer { match('^FileBrowser$' value('.SDATA.synolog@6574.synotype')); };" \
    >> ${SYSLOGNG_CONFIG_RECVRULE}
    echo "filter f_syno_webdavxfer { match('^Webdav$' value('.SDATA.synolog@6574.synotype')); };" \
    >> ${SYSLOGNG_CONFIG_RECVRULE}
    echo "filter f_syno_cifsxfer { match('^WinFileService$' value('.SDATA.synolog@6574.synotype')); };" \
    >> ${SYSLOGNG_CONFIG_RECVRULE}
    echo "filter f_syno_afpxfer { match('^MacFileService$' value('.SDATA.synolog@6574.synotype')); };" \
    >> ${SYSLOGNG_CONFIG_RECVRULE}

    echo "filter f_syno_transfer { filter(f_syno_ftpxfer) or filter(f_syno_fsxfer) or filter(f_syno_fbxfer) or filter(f_syno_webdavxfer) or filter(f_syno_cifsxfer) or filter(f_syno_afpxfer) };" \
    >> ${SYSLOGNG_CONFIG_RECVRULE}
}

IetfFilterProcess(){
    GeneralFilterProcess
    ConnectionFilterProcess
    TransferFilterProcess
}

IetfDestProcess(){
    local DB_PATH=
    local DB_PATH_SYS=
    local DB_PATH_CON=
    local DB_PATH_XFER=
    if [ "${SYSLOGNG_SERVER_ARCH_BYDEVICE}" -eq 1 ]; then
        DB_PATH="${SYSLOGNG_SERVER_ARCH_DEST}/\${HOST}/SYNOSYSLOGDB_\${HOST}.DB"
        DB_PATH_SYS="${SYSLOGNG_SERVER_ARCH_DEST}/\${HOST}/SYNOSYSLOGDB_\${HOST}_SYS"
        DB_PATH_CON="${SYSLOGNG_SERVER_ARCH_DEST}/\${HOST}/SYNOSYSLOGDB_\${HOST}_CON"
        DB_PATH_XFER="${SYSLOGNG_SERVER_ARCH_DEST}/\${HOST}/SYNOSYSLOGDB_\${HOST}_XFER"
    else
        DB_PATH="${SYSLOGNG_SERVER_ARCH_DEST}/SYNOSYSLOGDB__ARCH.DB"
        DB_PATH_SYS="${SYSLOGNG_SERVER_ARCH_DEST}/SYNOSYSLOGDB__ARCH_SYS"
        DB_PATH_CON="${SYSLOGNG_SERVER_ARCH_DEST}/SYNOSYSLOGDB__ARCH_CON"
        DB_PATH_XFER="${SYSLOGNG_SERVER_ARCH_DEST}/SYNOSYSLOGDB__ARCH_XFER"
    fi

    #main database
    sed "s=__SYNO_DB_PATH__=${DB_PATH}=; s=__SYNO_RULE_NAME__=all_device=" \
        "${SYSLOGNG_SERVER_TEMPLATE_FOLDER}/arch_dest_db.template" >> ${SYSLOGNG_CONFIG_RECVRULE}

    #structure data
    sed "s=__SYNO_DB_PATH__=${DB_PATH_SYS}=; s=__SYNO_RULE_NAME__=all_device=" \
        "${SYSLOGNG_SERVER_TEMPLATE_FOLDER}/arch_dest_db_general.template" >> ${SYSLOGNG_CONFIG_RECVRULE}

    sed "s=__SYNO_DB_PATH__=${DB_PATH_CON}=; s=__SYNO_RULE_NAME__=all_device=" \
        "${SYSLOGNG_SERVER_TEMPLATE_FOLDER}/arch_dest_db_connection.template" >> ${SYSLOGNG_CONFIG_RECVRULE}

    sed "s=__SYNO_DB_PATH__=${DB_PATH_XFER}=; s=__SYNO_RULE_NAME__=all_device=" \
        "${SYSLOGNG_SERVER_TEMPLATE_FOLDER}/arch_dest_db_transfer.template" >> ${SYSLOGNG_CONFIG_RECVRULE}
}

IetfDBLogProcess(){
    echo "log { source(s_syno_ietf_${RULE_NAME}); source(s_syno_ietf_${RULE_NAME}_6); flags(flow_control); };" \
    >> ${SYSLOGNG_CONFIG_RECVRULE}
}
IetfPipeLogProcess(){
    echo "log { source(s_syno_ietf_${RULE_NAME}); source(s_syno_ietf_${RULE_NAME}_6); destination(d_syno_dest_recv); flags(flow_control); };" \
    >> ${SYSLOGNG_CONFIG_RECVRULE}
}
IetfArchLogProcess(){
    #main database
    echo "log { source(s_syno_ietf_${RULE_NAME}); source(s_syno_ietf_${RULE_NAME}_6); destination(d_syno_arch_all_device_db); flags(flow_control); };" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}

    #structure data - general log
    echo "log { source(s_syno_ietf_${RULE_NAME}); source(s_syno_ietf_${RULE_NAME}_6); filter(f_syno_general); destination(d_syno_general_all_device_db); flags(flow_control); };" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}
    #structure data - connection log
    echo "log { source(s_syno_ietf_${RULE_NAME}); source(s_syno_ietf_${RULE_NAME}_6); filter(f_syno_connection); destination(d_syno_connection_all_device_db); flags(flow_control); };" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}
    #structure data - transfer log
    echo "log { source(s_syno_ietf_${RULE_NAME}); source(s_syno_ietf_${RULE_NAME}_6); filter(f_syno_transfer); destination(d_syno_transfer_all_device_db); flags(flow_control); };" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}
}
IetfLogProcess(){
    IetfDBLogProcess
    IetfPipeLogProcess
    IetfArchLogProcess
}

CustomSingleRuleGen() {
    if [ "yes" != ${RULE_ENABLE} ]; then
        return
    fi

    CustomSourceProcess
    CustomDestProcess
    CustomParserProcess
    CustomLogProcess
}
CustomSourceProcess() {
    local SOCKET_BUFFER=`awk '{print $3}' /proc/sys/net/ipv4/tcp_wmem`
    local PROTOCOL="udp"
    local TLS_RULE=""

    if [ -z "${SOCKET_BUFFER}" ]; then
        SOCKET_BUFFER="4194304"
    fi

    if [ "tcp" == "${RULE_PROTOCOL}" ]; then
        PROTOCOL="tcp"
        if [ "yes" == "${RULE_SSL}" ]; then
            PROTOCOL="tls"
            TLS_RULE="tls(key_file(\"/var/packages/LogCenter/target/service/conf/keys/ca.key\") cert_file(\"/var/packages/LogCenter/target/service/conf/keys/ca.crt\") peer_verify(optional-untrusted))"
        fi
    fi

    # process v4 source
    cat ${SYSLOGNG_TEMPLATE_SRC} |
    sed -e "s/__SYNO_SRCNAME__/cust_${RULE_NAME}/" \
        -e "s/__SYNO_SRCTYPE__/network/" \
        -e "s/__SYNO_ANYADDR__/\"0.0.0.0\"/" \
        -e "s/__SYNO_TRANSPORT__/${PROTOCOL}/" \
        -e "s/__SYNO_PORT__/${RULE_PORT}/" \
        -e "s/__SYNO_IPV__/4/" \
        -e "s=__SYNO_CA_SETTING__=${TLS_RULE}=" \
        -e "s/__SYNO_SOCKBUF__/${SOCKET_BUFFER}/" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}

    # process v6 source
    cat ${SYSLOGNG_TEMPLATE_SRC} |
    sed -e "s/__SYNO_SRCNAME__/cust_${RULE_NAME}_6/" \
        -e "s/__SYNO_SRCTYPE__/network/" \
        -e "s/__SYNO_ANYADDR__/\"::\"/" \
        -e "s/__SYNO_TRANSPORT__/${PROTOCOL}/" \
        -e "s/__SYNO_PORT__/${RULE_PORT}/" \
        -e "s/__SYNO_IPV__/6/" \
        -e "s=__SYNO_CA_SETTING__=${TLS_RULE}=" \
        -e "s/__SYNO_SOCKBUF__/${SOCKET_BUFFER}/" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}
}
CustomDestProcess(){
    local DB_PATH=
    if [ "${SYSLOGNG_SERVER_ARCH_BYDEVICE}" -eq 1 ]; then
        DB_PATH="${SYSLOGNG_SERVER_ARCH_DEST}/\${HOST}/SYNOSYSLOGDB_\${HOST}.DB"
    else
        DB_PATH="${SYSLOGNG_SERVER_ARCH_DEST}/SYNOSYSLOGDB__ARCH.DB"
    fi

    sed "s=__SYNO_DB_PATH__=${DB_PATH}=; s=__SYNO_RULE_NAME__=all_device=" \
        "${SYSLOGNG_SERVER_TEMPLATE_FOLDER}/arch_dest_db.template" >> ${SYSLOGNG_CONFIG_RECVRULE}
}
CustomParserProcess() {
    local PATTERN_DB_FILE=${RULE_NAME}".xml"
    local MD5=`date | openssl md5 | awk '{print $2}'`

    # 1. generate pattern db file, put it in "${LOGCENTER_TARGET}/usr/syno/etc/synosyslog/patterndb/<name>.xml"
    #    the pattern needs to overwrite following macro for Synology Log Center if necessary
    #
    #      $YEAR $MONTH $DAY $HOUR $MIN $SEC
    #      $HOSTNAME $PROGRAM $FACILITY $PRIORITY $MSGONLY
    #

    rm -rf ${SYSLOGNG_FOLDER_PATTERN_DB_XML}${PATTERN_DB_FILE} > /dev/null 2>&1
    mkdir -p ${SYSLOGNG_FOLDER_PATTERN_DB_XML}
    touch ${SYSLOGNG_FOLDER_PATTERN_DB_XML}${PATTERN_DB_FILE}
    cat ${SYSLOGNG_TEMPLATE_PATTERN_DB} |
    sed -e "s/__SYNO_ID__/${RULE_NAME}/" \
        -e "s/__SYNO_PATTERNDB_RULE__/${RULE_FORMAT}/" \
        -e "s/__SYNO_MD5__/${MD5}/" \
        >> ${SYSLOGNG_FOLDER_PATTERN_DB_XML}${PATTERN_DB_FILE}

    # 2. create rule to use the pattern
    #
    #      parser p_parser_<name> {
    #          db_parser(
    #              file("${LOGCENTER_TARGET}/usr/syno/etc/synosyslog/patterndb/<name>.xml")
    #          );
    #      };
    #

    cat ${SYSLOGNG_TEMPLATE_PARSER} |
    sed -e "s/__SYNO_NAME__/${RULE_NAME}/" \
        -e "s/__SYNO_PATTERNDB_FILE__/${PATTERN_DB_FILE}/" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}
}

CustomLogProcess() {
    echo "log { source(s_syno_cust_${RULE_NAME}); source(s_syno_cust_${RULE_NAME}_6); parser(p_parser_${RULE_NAME}); flags(flow_control); };" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}
    echo "log { source(s_syno_cust_${RULE_NAME}); source(s_syno_cust_${RULE_NAME}_6); parser(p_parser_${RULE_NAME}); destination(d_syno_dest_recv); flags(flow_control); };" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}
    echo "log { source(s_syno_cust_${RULE_NAME}); source(s_syno_cust_${RULE_NAME}_6); parser(p_parser_${RULE_NAME}); destination(d_syno_arch_all_device_db); flags(flow_control); };" \
        >> ${SYSLOGNG_CONFIG_RECVRULE}
}

