#!/bin/sh

###############################################################
#   This script should be hooked when invoking SYNOShareSet()
#   Usable environment variable:
#       ORIGIN_SHARE_NAME, SHARE_NAME, ORIGIN_SHARE_PATH, SHARE_PATH,
#       ORIGIN_SHARE_STATUS, SHARE_STATUS
#
#   Note: The script should bear to be invoked multiple times
###############################################################

#Include common scripts
LOGCENTER_TARGET=/var/packages/LogCenter/target
FILE_SYSLOGNG_INC_SCRP=${LOGCENTER_TARGET}/service/sharehook/SyslogShareCommon.sh
. ${FILE_SYSLOGNG_INC_SCRP}
. /var/packages/LogCenter/scripts/pkg_utils

#At begining, acquire package settings
PackageInfoGet;

LocationChangeCheck(){
    # Only hooked when share name or share path changed
    if [ "${ORIGIN_SHARE_NAME}" != "${SHARE_NAME}" ] || [ "${ORIGIN_SHARE_PATH}" != "${SHARE_PATH}" ] || 
       [ "${ORIGIN_SHARE_STATUS}" = "decrypt" -a "${SHARE_STATUS}" = "normal" ] ||
       [ "${ORIGIN_SHARE_STATUS}" = "normal" -a "${SHARE_STATUS}" = "decrypt" ]; then
        NEED_HOOK=1;
    else
        NEED_HOOK=0;
    fi
}

case $1 in
    --sdk-mod-ver)
        #Print SDK support version
        echo ${SYSLOGNG_PKG_MODVER};
    ;;
    --name)
        #Print package name
        echo ${SYSLOGNG_PKG_NAME};
    ;;
    --pkg-ver)
        #Print package version
        echo ${SYSLOGNG_PKG_VERSION};
    ;;
    --vendor)
        #Print package vendor
        echo ${SYSLOGNG_PKG_VENDOR};
    ;;
    --pre)
        if ! IsServiceShare "${ORIGIN_SHARE_NAME}" ; then
            exit
        fi
        if [ ${SHARE_ENABLE} -eq 1 ] ; then
            exit
        fi
        RecordStatusAndShutDown
    ;;
    --post)
        #Actions after share set
        if ! IsServiceShare "${ORIGIN_SHARE_NAME}" ; then
            exit
        fi

        #Check hook condition
        LocationChangeCheck;
        if [ "1" != "${NEED_HOOK}" ]; then
            exit;
        fi

        #Check action success or not
        if [ "0" = "${RESULT}" ]; then
            SyslogServiceLocSet;
            CheckStatusAndStartUp
            SyslogMailShareRename;
        fi

        ################################################
        # PS: Service is resumed/disabled by service framework
        ################################################
    ;;
    *)
        echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
    ;;
esac

